/**
 * $id$
 * $Name:  $
 *
 * Device driver for curses system.
 *
 * $log$
 */
static const char rcsinfo[] = "$Id: adcurses.cpp,v 1.2 2008/05/13 22:59:42 mviara Exp $";

#include "ansi.hpp"

#include <curses.h>


CAdapterCurses::CAdapterCurses()
{
	static int colors[] =
	{
		COLOR_BLACK,
		COLOR_BLUE,
		COLOR_GREEN,
		COLOR_CYAN,
		COLOR_RED,
		COLOR_YELLOW,
		COLOR_MAGENTA,
		COLOR_WHITE
	};

	
	initscr();
	start_color();
	raw();
	noecho();
	nonl();
	intrflush(stdscr,FALSE);
	keypad(stdscr,TRUE);

	for (int i = 0 ; i < 8 ; i++)
		for (int j = 0 ; j < 8 ; j++)
			init_pair(i*8+j+1,colors[i],colors[j]);
	numRows = LINES;
	numCols = COLS;
	
}

void CAdapterCurses::Locate(int pos)
{
	int y,x;

	y = pos / COLS;
	x = pos % COLS;
	move(y,x);

}

void CAdapterCurses::SetColor(unsigned col)
{


	if (col != curColor)
	{
		curColor = col;
		int pair =(curColor & 7) * 8 + ((curColor >> 4) & 7);
		attrset(COLOR_PAIR(pair+1));
		if (curColor & 8)
			attron(A_BOLD);
	}


}

void CAdapterCurses::Print(int pos,char ch,int num)
{
	int c = ch & 0xff;
	
	Locate(pos);

	if (c > 127)
	{
	switch (c)
	{
		case	217:
			c = ACS_LRCORNER;
			break;
		case	218:
			c = ACS_ULCORNER;
			break;
		case	179:
			c = ACS_VLINE;
			break;
		case	191:
			c = ACS_URCORNER;
			break;
		case	192:
			c = ACS_LLCORNER;
			break;
		case	193:
			c = ACS_BTEE;
			break;
			
		case	0xc4:
			c = ACS_HLINE;
			break;
		default:
			c = ACS_DIAMOND;
			break;
	}
	}

	while (num--)
		addch(c);
	//hline(ch,num);
	//refresh();
}

CAdapterCurses::~CAdapterCurses()
{
	endwin();
}


int CAdapterCurses::Kbhit()
{
	refresh();
	nodelay(stdscr,TRUE);
	int c = getch();
	nodelay(stdscr,FALSE);

	if (c != ERR)
	{
		ungetch(c);
		return 1;
	}

	return 0;

}

unsigned CAdapterCurses::Getch()
{
	int altPressed = 0;

	
	int c =  getch();

	if (c == 27)
	{
		int c1;
		
		nodelay(stdscr,TRUE);
		for (int i = 0 ; i < 10 ; i++)
		{
			if ((c1 = getch()) != ERR)
				break;
			Sleep(10);
		}
		nodelay(stdscr,FALSE);
		if (c1 != ERR)
		{
			c = c1;
			altPressed = 1;
		}
	}

	switch (c)
	{
		case	'a':
		case	'A':
			return altPressed ? 0x1e00 : c;
		case	'b':
		case	'B':
			return altPressed ? 0x3000 : c;
		case	'c':
		case	'C':
			return altPressed ? 0x2e00 : c;
		case	'q':
		case	'Q':
			return altPressed ? 0x1000 : c;
		case	'x':
		case	'X':
			return altPressed ? 0x2d00 : c;
		case	'M' - '@':
			return altPressed ? 0x3201 : c;
		case	'C' - '@':
			return altPressed ? 0x2e01 : c;
		case	'I' - '@':
			return altPressed ? 0x1701 : c;
		case	'X' - '@':
			return altPressed ? 0x2d01 : c;
		case	KEY_UP:
			return 0x4800;
		case	KEY_DOWN:
			return 0x5000;
		case	KEY_LEFT:
			return 0x5200;
		case	KEY_RIGHT:
			return 0x5400;
		case	KEY_F(1):
		case	KEY_F(2):
		case	KEY_F(3):
		case	KEY_F(4):
		case	KEY_F(5):
		case	KEY_F(6):
		case	KEY_F(7):
		case	KEY_F(8):
		case	KEY_F(9):
		case	KEY_F(10):
			c = c - KEY_F(1);
			return altPressed ? 0x6800 + c*256  : 0x3b00 + c*256 ;
		case	KEY_NPAGE:
			return 0x5100;
			
		case	KEY_PPAGE:
			return 0x4900;
			
		default:
			if (c > 255)
				c = 13;
			return c;
	}
	return c;
}
