/**
 *
 * $Id: ansi.hpp,v 1.19 2008/05/13 17:46:19 mviara Exp $
 * $Author: mviara $
 * $Name:  $
 *
 * $Log: ansi.hpp,v $
 * Revision 1.19  2008/05/13 17:46:19  mviara
 * Fixed bug in Clear to EOL
 *
 * Revision 1.18  2008/05/13 17:30:03  mviara
 *
 * First beta version for Linux system.
 *
 * Revision 1.17  2007/12/17 22:53:50  mviara
 * Changed bell to short sound.
 *
 * Revision 1.16  2007/10/26 01:33:12  mviara
 * Added support for serial mode.
 *
 * Revision 1.15  2005/11/21 19:00:02  mviara
 * Added supporto to report terminal id and windows title.
 *
 * Revision 1.14  2005/05/18 11:07:55  mviara
 * Added support for  RFC1093 (Windows size)
 *
 * Revision 1.13  2005/04/19 14:05:37  mviara
 * Added supporto save restore attribute with ESC7 / ESC8
 *
 * Revision 1.12  2005/04/03 08:27:47  mviara
 * Added support for copy paste text.
 *
 * Revision 1.11  2005/04/02 07:26:29  mviara
 * Added partial support for copy/paste text.
 *
 * Revision 1.10  2005/03/28 12:53:09  mviara
 * Added support for xterm, fixed bug in attributr at begin of line.
 *
 * Revision 1.9  2005/01/24 17:31:21  mviara
 * Fixed bug in handling ESC[m
 *
 * Revision 1.8  2004/12/06 22:12:13  mviara
 * Removed unused code.
 *
 * Revision 1.7  2004/12/06 20:07:33  mviara
 * Removed old unused code.
 *
 * Revision 1.6  2004/12/06 07:31:05  mviara
 * Improved performance.
 *
 * Revision 1.5  2004/11/30 20:05:28  mviara
 * Version 2.02
 *
 * Revision 1.4  2004/11/10 18:40:17  mviara
 * Version 2.01
 *
 * Revision 1.3  2004/07/12 15:59:35  mviara
 * Removed same rem, added supporto for Visual C and Watcom C.
 *
 * Revision 1.2  2004/07/10 13:59:58  mviara
 * Add Supporto for connection time.
 *
 * Revision 1.1.1.1  2004/07/10 12:33:21  mviara
 * Migrated from RCS
 *
 * Revision 1.1  2001/08/08 16:14:42  VM
 * Initial revision
 *
 * 
 */
#ifdef __LINUX__
#include <assert.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <fcntl.h>
#include <unistd.h>
#include <time.h>
#include <ctype.h>
#include <errno.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/times.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <sys/ioctl.h>

#endif

#ifdef __WIN32__
#include <windows.h>
#include <dos.h>
#include <assert.H>
#include <stdio.h>
#include <string.h>
#include <conio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <fcntl.h>
#include <io.h>
#include <time.h>
#include <ctype.h>
#endif

#include <assert.h>


#define versionString	"Beta 3.00"
#define	copyright	"ANSI32 Vers. "versionString" - By Mario Viara - mario@viara.cn"
void AnsiPrintf(char * , ...);
void AnsiPutchar(char c);
void AnsiError(char *, ...);


typedef union
{
	unsigned flags;
	struct
	{
		unsigned caps:1;
		unsigned nums:1;
		unsigned shift:1;
		unsigned alt:1;
		unsigned ctrl:1;
		unsigned altgr:1;
	} flag;
} KSTATE;
   
//
// Struttura per definire un carattere nel buffer video
//
typedef struct
{
	unsigned char chr;		// carattere
	unsigned char  color;		// attributo
} SCHAR;




//
// Struttura per gli adatattori video
//

class CAdapter
{
protected:
	short		numRows;	// numero di righe
	short		numCols;	// numero di colonne
	unsigned	curColor;	// colore corrente
	KSTATE		kstate;
public:
	CAdapter();
	virtual char * Name();		// nome del driver
	virtual int  GetColor() {return curColor;};
	virtual void SetColor(unsigned col);
	virtual void Locate(int pos)=0;
	virtual void PrintBuffer(int pos,char * buffer,int num);
	virtual void Print(int pos,char  ch,int num)=0;
	virtual int Kbhit() = 0;		// torna 1 se c'e' un tasto
	virtual unsigned Getch() =0;	// legge un tasto
	virtual void HideCursor();	// nasconde il cursore
	virtual void ShowCursor();	// visualizza il cursore
	virtual void BigCursor();
	virtual void LittleCursor();
	void Print(int pos,char *buffer);
	int Rows() {return numRows;}	// torna il numero di righe
	int Cols() {return numCols;}	// torna il numero di colonne
	int Size() {return numCols*numRows;};
	virtual KSTATE GetState();
};

class CAdapBIOS : public CAdapter
{
private:
	virtual char * Name();
	virtual void Locate(int pos);
	virtual void Print(int pos,char  ch,int num);
public:
	CAdapBIOS();

};

class CAdaGRAPH : public CAdapter
{
private:
	unsigned oldCurColor;
	virtual char * Name();
	virtual void Locate(int pos);
	virtual void Print(int pos,char  ch,int num);
public:
	CAdaGRAPH();
};

#ifdef __LINUX__
class CAdapterCurses : public CAdapter
{
	virtual void Locate(int pos);
	virtual void Print(int pos,char  ch,int num);
	virtual int Kbhit();
	virtual unsigned Getch();
	virtual void SetColor(unsigned col);
	
public:
	CAdapterCurses();
	virtual ~CAdapterCurses();
};

#endif

#ifdef __WIN32__
class CAdapterNT : public CAdapter
{
	CONSOLE_CURSOR_INFO	cursorInfo;
	unsigned		oldAttribute;
	unsigned Getkey(int kbdtype,int padnum);
private:
	int	kbdType;
	int	padnum;
	HANDLE hStdOut;
	HANDLE hStdIn;
	DWORD  oldConsoleOutMode;
	DWORD  oldConsoleInMode;
	unsigned key;
	virtual char * Name();
	virtual void SetColor(unsigned col);
	virtual void Locate(int pos);
	virtual void Print(int pos,char  ch,int num);
	virtual void PrintBuffer(int pos,char *buffer,int num);
	virtual int Kbhit();
	virtual unsigned Getch();
	virtual void HideCursor();
	virtual void ShowCursor();
	virtual void BigCursor();
	virtual void LittleCursor();

	void	SetAttribute();

public:
	CAdapterNT(int keyboard,int p);
	~CAdapterNT();
};
#endif

class CDevice
{
	protected:
	char	proxyName[256];
	int	flagProxy;
	
public:
	CDevice();
	void	SetProxy(char *);
	virtual int  IsConnected(int code);
	virtual void Deactivate(int code);	// deattiva la session
	virtual int Connect(int code,char *id,int row,int col);		// connette la sessione
	virtual void Disconnect(int code);	// disconnette la sessione
	virtual int Read(char *buffer,int maxSize)=0;
	virtual int Write(char *buffer,int size)=0;
	virtual char * Name() ;			// torna il nome del driver
};

#ifdef __WIN32__
class CDeviceCOM : public CDevice
{
private:
	HANDLE			hCom;
	DCB			dcb;
	COMMTIMEOUTS		ctout;
	char			parameter[256];
	int			firstRead;
public:
	int GetParameter(int index,char *buffer,char s);
	int GetParameter(int index,char *buffer);
	int GetParameter(int index);
	CDeviceCOM(char *port);
	virtual int Read(char *buffer,int maxSize);
	virtual int Write(char *buffer,int size);
	virtual char * Name() ;
	virtual int	IsConnected(int code);
	virtual int	Connect(int code,char *id,int row,int col);
	
};
#endif

class CDeviceECHO : public CDevice
{
	char buffer[256];
	int size;
public:
	CDeviceECHO();
	virtual int Read(char *buffer,int maxSize);
	virtual int Write(char *buffer,int size);
	virtual char * Name() ;
};

class CDeviceINT14 : public CDevice
{
	int port;
public:
	CDeviceINT14() {port=0;};
	virtual int Read(char *buffer,int maxSize);
	virtual int Write(char *buffer,int size);
	virtual char * Name() ;
};



	
class CAnsi
{
public:
	class CScreen
	{
		public:
		enum
		{
			KEYBUF = 127,		// Deve essere 2^n - 1
			KEYLEN = 16,		// lunghezza tasti funzione
			FCNKEY= 71
		};

		KSTATE	state;
		int	numRow,numCol;
		SCHAR * buffer;		
		int	session;
		void	(CAnsi::*DisplayChar)(unsigned char);

		// Char used for CSI
		int     csi;

		// Character parameter
		char	pt[256];
		int	ptlen;
		
		// Current parameter not entered (CSI)
		int	nopar;

		// Number of CSI parameter
		int	numpar;

		// Parameter array Vt520 specification
		int	par[16];

		// Parameter pointer
		int *	parptr;		
		int	ps;			/* pointer scritura buffer tastiera	*/
		int	pl;		/* pointer lettura buffer tastiera	*/
		unsigned char	keybuf[KEYBUF+1];
		unsigned char	keys[FCNKEY][KEYLEN]; // tasti funzione		*/
		unsigned char	lenght[FCNKEY];	/* lunghezze tasti funzione	*/
		
		struct
		{
			int	gvt100:1;
			int	autoWrap:1;
			int	wrapped:1;
			int	scrolloff:1;
			int	initialized:1;
		};

		typedef struct
		{
			unsigned bnormal,fnormal;
			unsigned breverse,freverse;
			unsigned obcolor,ofcolor;
			unsigned bcolor,fcolor;
			unsigned bgraphics,fgraphics;
			unsigned reverse:1;
			unsigned blink:1;
			unsigned hi:1;
		} ScreenAttribute;

		char	 * statusLine;
		long	lptCounter;
		SCHAR   *lptVideoBuffer;
		unsigned lptCursor;
		char	lptTmp[256];
		FILE	*fdlpt;
		int	printchar;	/* caratteri da stampare ma non ancora	*/
						/* stampati perche' uguali alla fine	*/
						/* stampa.				*/
		int	bscroll;	/* riga iniziale scroll			*/
		int	escroll;	/* riga finale scroll			*/

		ScreenAttribute	attribute,oldAttribute;
		int	cursorOn;
		int	curPos;
		int	oldCurPos;
		unsigned	newColor;
		unsigned	curColor;
		unsigned	orMask;
		void SetColor(unsigned col) {curColor = col;};
		void Locate(int pos) {curPos = pos;};
		void Print(char ch,int num);
		void Print(int pos,char  ch,int num){Locate(pos);Print(ch,num);};
		CScreen(int row,int col);
		~CScreen();
		void Default();
	};

	enum
	{
		MAXSCREENS = 10,
	        KEYSCROLLOFF = 0xff00,
		KEYSCROLLON  = 0xff01,			     
	};

	enum EMode
	{
		Ansix364,
		Vt100,
		Linux,
		Xterm
	};

	char * copyText;
	
private:
	friend void AnsiPrintf(char *,...);
	friend void AnsiError(char *,...);
	friend void AnsiPutchar(char);
	
	static CAnsi * defaultAnsi;
	SCHAR	* tmpBuffer;
	EMode	terminalMode;
	int	doubleBuffer;
	int	selectPrinter;
	long	deviceRead;
	long	deviceWrite;
	CScreen * curScr;
	CScreen * screens[MAXSCREENS+1];
	time_t	screenTime[MAXSCREENS+1];
	unsigned startConnect[MAXSCREENS+1];
	unsigned endConnect[MAXSCREENS+1];
	CAdapter * adapter;
	CDevice  * device;
	int	flagSwitch;						// Flag switch screen attivo
	int	flagExit ;
	int	fdCapture;
	int	fdFromFile;
	int	numRows;
	int	numCols;
	int	screenSize;
	char	printerName[64];
	static  unsigned char tabColor[];
public:
	unsigned char Ansi2Vga(unsigned char color);
	unsigned char Vga2Ansi(unsigned char color);
	
	void HideCursor();
	void ShowCursor();
	void Macro();
	void Capture();
	void CopyText();
	void PasteText();
	CAnsi(CAdapter * a,CDevice *d,EMode  ,int=0,int=0);
	~CAnsi();
	void SetColor();
	void SetColor(unsigned color);
	void ResetScreen();
	int  ChangeScreen();
	void AddScreen(int s);
	int  SwitchScreen(int s);
	void ActivateScreen(int s);
	void ConnectScreen(int s);
	void DeactivateScreen(int s);
	void DeleteScreen();
	void Main();
        void Putchar(char);
	void Outstr(char *buffer,int len);
	void Outstr(char * buffer) {Outstr(buffer,strlen(buffer));};
	void CalcAttribute();
	void PrinterOutput(unsigned char c);
	void DisplaySpecial(unsigned char c);
	void SelectPrinter(char*);
	void StartPrinter();
	void StopPrinter();
	void DisplayPrinter(unsigned char c);
	void DisplayNormal(unsigned char c);
	void DisplayEscape(unsigned char c);
	void DisplayCsi(unsigned char c);
	void DisplayAdditional(unsigned char c);
	void DisplayFcnkey(unsigned char c);
	void DisplayPrgkey(unsigned char c);
	void DisplayVt100Mode(unsigned char c);
	void ProcessAttribute(int n,int num);
	void CsiAttribute();
	void OutChar(unsigned char c);
	void Bell();
	void BackSpace();
	void Tab();
	void BackTab(int ) {}; //bug
	void LineFeed();
	void CarriageReturn();
	void ScrollUp(int n);
	void ScrollDn(int n);
	void VideoCopy(int source,int dest,int count);
	void VideoGet(int pos,int n,SCHAR * buffer);
	void VideoClear(int r,int c,int nr,int nc);
	void VideoPut(int pos,int n,SCHAR * buffer);
	void VideoSet(int pos,char c,int n);
	void Locate();
	void StartEscape() {curScr->DisplayChar = &CAnsi::DisplayEscape;};
	void StartCsi() ;
	int GetPar(char c);
	void Inschar(int col,int no,int eol)	;
	void Delchar(int col,int no,int eol)	;
	void KeybInsert(unsigned k);
	void KeybInsert(char *);
	void KeybInput();
	int  KeybStatus();
	int  KeybRead();
	int  Read(char *,int);
	int  Write(char * ,int);
	void Info();
	void PrintScreen();
	void SetPrinterName(char *);
	void SetSwitch(int flag);
	char * TermId();
	void StatusLine(char *fmt,...);
	void ShowStatusLine();
};


class CDeviceTelnet : public CDevice
{
	typedef struct
	{
		int	socket;
		int	echoTX;
		int	echoRX;
		int	isValid;
		char	bufferIn[512];
		int	readBufferIn;
		int	numBufferIn;
	} STelnetSession;
	
	char * termType;
	int	row,col;
	char hostName[64];
	STelnetSession	telnetSession[CAnsi::MAXSCREENS];
	STelnetSession *curSession;
	int session;
	enum
	{
		TELOPT_ECHO = 1,
		TELOPT_TTYPE = 24,
		TELOPT_NAWS  = 31,
		DO = 253,
		WILL = 251,
		WONT = 252,
		DONT = 254,
		IAC = 255,
		SB = 250,
		SE = 240,
	};
	int WriteRaw(unsigned char *buffer,int size);	
	void Send3(int n1,int n2,int n3);
	void Dont(int c) {Send3(IAC,DONT,c);};
	void Will(int c) {Send3(IAC,WILL,c);};
	void Wont(int c) {Send3(IAC,WONT,c);};
	void Do(int c)   {Send3(IAC,DO,c);};
	int  RecvChar();
	void SBReceived();
	void WILLReceived();
	void WONTReceived();
	void DOReceived();
	int  IACReceived();
	void DONTReceived();
	int CheckSocket();
public:
	CDeviceTelnet(char * hostname);
	virtual ~CDeviceTelnet();
	virtual int	IsConnected(int code);
	virtual int		Connect(int code,char *id,int row,int col);
	virtual void Deactivate(int code);
	virtual void Disconnect(int code);
	virtual int Read(char *buffer,int maxSize);
	virtual int Write(char *buffer,int size);
	virtual char * Name() ;
};


#define	APPCODE	0x19980120
#define APPNAME "ansi32"
	
#ifdef __LINUX__
unsigned long GetTickCount();
int itoa(int,char *,int base);
void Sleep(int n);
#define closesocket	close
#define	ioctlsocket	ioctl
#define O_BINARY	0
#endif
