/**
 * $Id: devcom.cpp,v 1.1 2007/10/26 01:33:12 mviara Exp $
 * $Author: mviara $
 * $Name:  $
 *
 * Device for serial port.
 *
 * $Log: devcom.cpp,v $
 * Revision 1.1  2007/10/26 01:33:12  mviara
 * Added support for serial mode.
 *
 *
 */
#include "ansi.hpp"

static char rcsinfo[] = "$Id: devcom.cpp,v 1.1 2007/10/26 01:33:12 mviara Exp $";

CDeviceCOM::CDeviceCOM(char *parameter)
{
	strcpy(this->parameter,parameter);
	hCom = INVALID_HANDLE_VALUE;
	firstRead = 1;

}

int CDeviceCOM::IsConnected(int code)
{
	return hCom == INVALID_HANDLE_VALUE ? 0 : 1;
	
}

int CDeviceCOM::GetParameter(int index,char *buffer,char s)
{
	char *p = parameter;
	int i;
	
	for (i = 0 ; i < index ; i++)
	{
		p = strchr(p,s);
		if (p == 0)
			return 1;
		p++;
	}

	
	for (i = 0 ; p[i] ; i++)
	{
		if (p[i] == s)
			break;
		buffer[i] = p[i];
	}

	buffer[i] = 0;
	return 0;
	
}

int CDeviceCOM::GetParameter(int index,char *buffer)
{
	if (GetParameter(index,buffer,':'))
		if (GetParameter(index,buffer,','))
			return 1;

	return 0;
}

int CDeviceCOM::GetParameter(int index)
{
	char tmp[256];

	if (GetParameter(index,tmp))
		return -1;

	return atoi(tmp);
				
}


int CDeviceCOM::Connect(int code,char *id,int row,int col)
{
	//char *port = "\\\\.\\COM10";
	char port[32] ;
	int baud = 115200;
	int bits = 8;
	int stop = 1;
	char parity = 'N';
	char tmp[32];


	if (hCom != INVALID_HANDLE_VALUE)
		return 0;

	strcpy(port,"COM1");

	GetParameter(0,port);
	if ((baud = GetParameter(1)) < 0)
		baud = 9600;
	if (GetParameter(2,tmp))
		parity = 'N';
	else
		parity = tmp[0];

	if ((bits = GetParameter(3)) <= 0)
		bits = 8;
	if ((stop = GetParameter(4)) <= 0)
		stop = 1;

	AnsiPrintf("Opening %s:%d,%c,%d,%d ....",
	       port,baud,parity,bits,stop);
	
	hCom = CreateFile(port
			  , GENERIC_READ | GENERIC_WRITE
			  , 0
			  , NULL
			  , OPEN_EXISTING
			  , FILE_FLAG_NO_BUFFERING
			  , NULL
			 );

	if (hCom == INVALID_HANDLE_VALUE)
	{
		AnsiPrintf("\n");
		return 1;
	}

	ctout.ReadIntervalTimeout		= 0;
	ctout.ReadTotalTimeoutMultiplier	= 0;
	ctout.ReadTotalTimeoutConstant		= 0;
	ctout.WriteTotalTimeoutMultiplier	= 0;
	ctout.WriteTotalTimeoutConstant		= 0;

	SetCommTimeouts(hCom,&ctout);

	GetCommState(hCom, &dcb);
	dcb.BaudRate = baud;
	dcb.ByteSize = (BYTE)bits;
	if (stop == 2)
		dcb.StopBits = 2;
	else
		dcb.StopBits = 0;

	dcb.fBinary = 1;

	switch (parity)
	{
		case	'n':
		case	'N':
			dcb.fParity = 0;
			dcb.Parity = 0;
			break;
		case	'e':
		case	'E':
			dcb.fParity = 1;
			dcb.Parity = 2;
			break;
		case	'o':
		case	'O':
			dcb.fParity = 1;
			dcb.Parity = 1;
			break;
	}

	dcb.fOutxCtsFlow = 0;
	dcb.fOutxDsrFlow = 0;
	SetCommState(hCom, &dcb);


	return 0;
}

char * CDeviceCOM::Name()
{
	return "COM $Revision: 1.1 $";
}

int CDeviceCOM::Write(char * p,int len)
{
	if (hCom == INVALID_HANDLE_VALUE)
		return -1;

	DWORD bytesWritten = 0;
	ctout.WriteTotalTimeoutConstant		= 5000;
	SetCommTimeouts(hCom,&ctout);

	WriteFile(hCom, p,len,&bytesWritten,NULL);

	return bytesWritten;
}

int CDeviceCOM::Read(char * p,int maxsize)
{
	if (hCom == INVALID_HANDLE_VALUE)
		return -1;

	if (firstRead && maxsize)
	{
		firstRead = 0;
		*p = 13;
		return 1;
	}
	
	DWORD bytesRead = 0;
	ctout.ReadTotalTimeoutConstant		= 10;

	SetCommTimeouts(hCom,&ctout);

	if (ReadFile(hCom,p,maxsize,&bytesRead,NULL))
	{
		if (bytesRead > 0)
			return bytesRead;
	}

	return 0;
}

